import time

from langgraph.graph import StateGraph, END
from typing_extensions import TypedDict
from llm.build_model import qwen2
import os
from langchain_core.output_parsers import StrOutputParser
from config.config import log
from common.data_utils import build_params
physics_knowledge_set = set()
math_knowledge_set = set()
with open(os.environ.get("PHYSICS_KNOWLEDGES_FILE_PATH"), 'r', encoding="utf8") as f:
    for knowledge in f:
        knowledge = knowledge.strip()
        physics_knowledge_set.add(knowledge)

with open(os.environ.get("MATH_KNOWLEDGES_FILE_PATH"), 'r', encoding="utf8") as f:
    for knowledge in f:
        knowledge = knowledge.strip()
        math_knowledge_set.add(knowledge)
chain = qwen2 | StrOutputParser()

class TopicGraphState(TypedDict):
    """
    表示topic图的状态
    Attributes:
        topic_content: topic content
        label_value: label value
        subject_id: 学科id
    """
    topic_content: str
    label_value: str
    subject_id: int

workflow = StateGraph(TopicGraphState)

#生成标签
def generate_labels(state):
    topic_content = state["topic_content"]
    log.debug("学科id:"+str(state["subject_id"]))
    label_value = chain.invoke(topic_content, subject_id=state["subject_id"])
    return {"label_value": label_value, "topic_content": topic_content}

#标签过滤
def filter_label_func(label_value: str, knowledge_set: set):
    filter_labels = []
    labels = list(set(label_value.split(",")))
    for label in labels:
        if label in knowledge_set:
            filter_labels.append(label)

    if len(filter_labels) == 0:
        return ""
    else:
        return ",".join(filter_labels)

#路由
def route_topic(state):
    label_value = state["label_value"]
    if len(label_value) == 0:
        return END
    if state["subject_id"] == 12:
        return "physics_knowledge_check"
    if state["subject_id"] == 3:
        return "math_knowledge_check"
    return END

#物理知识点标签处理
def physics_knowledge_check(state):
    label_value = state["label_value"]
    labels = label_value.split(",")
    filter_labels = []
    #1. 关键词匹配，添加知识点
    if "查理定律" in state["topic_content"]:
        filter_labels.append("气体的等容变化")
    if "盖-吕萨克定律" in state["topic_content"] or "盖吕萨克定律" in state["topic_content"]:
        filter_labels.append("气体的等压变化")

    #2. 过滤知识点
    error_label = []
    for label in labels:
        if (label == "x-t图象" or label == "v-t图象") and ("匀加速" in state["topic_content"] or "匀减速" in state["topic_content"]):
            filter_labels.append("运动图象")
            continue
        if label == "追及相遇问题" and "自由落体" in state["topic_content"]:
            continue
        if label == "实验：验证机械能守恒定律" and "机械能" not in state["topic_content"]:
            continue
        if label == "气体的等容变化" and ("等容" not in state["topic_content"] or "查理定律" not in state["topic_content"]):
            continue
        if label == "实验：测定金属的电阻率":
            filter_labels.append("测定金属的电阻率")
            continue
        if label in physics_knowledge_set:
            filter_labels.append(label)
        else:
            error_label.append(label)
    if len(filter_labels) == 0:
        topic_content = state["topic_content"]. \
            replace("，给出考察的知识点。", "，请给出考察的正确的知识点。输出知识点的个数请限制在2个之内")
        topic_content += "\n请不要输出以下知识点:\n" + ",".join(error_label)
        label_value = chain.invoke(topic_content, subject_id=state["subject_id"])
        label_value = filter_label_func(label_value, physics_knowledge_set)
        filter_labels = label_value.split(",")

    #3. 如果知识点列表为空，使用规则匹配
    if len(filter_labels) == 0:
        if "等温变化" in state["topic_content"]:
            filter_labels.append("玻意耳定律")
    label_value = ",".join(list(set(filter_labels)))
    return {"label_value": label_value, "topic_content": state["topic_content"]}

#数学知识点标签处理
def math_knowledge_check(state):
    label_value = state["label_value"]
    labels = label_value.split(",")
    filter_labels = []
    #1. 关键词匹配，添加知识点
    #2. 过滤知识点
    error_label = []
    for label in labels:
        if label in math_knowledge_set:
            filter_labels.append(label)
        else:
            error_label.append(label)
    if len(filter_labels) == 0:
        topic_content = state["topic_content"]. \
            replace("，给出考察的知识点。", "，请给出考察的正确的知识点。输出知识点的个数请限制在2个之内")
        topic_content += "\n请不要输出以下知识点:\n" + ",".join(error_label)
        label_value = chain.invoke(topic_content, subject_id=state["subject_id"])
        label_value = filter_label_func(label_value, math_knowledge_set)
        filter_labels = label_value.split(",")

    #3. 如果知识点列表为空，使用规则匹配
    # if len(filter_labels) == 0:
    #     if "等温变化" in state["topic_content"]:
    #         filter_labels.append("玻意耳定律")
    label_value = ",".join(list(set(filter_labels)))
    return {"label_value": label_value, "topic_content": state["topic_content"]}


#定义节点
workflow.add_node("generate", generate_labels)
workflow.add_node("physics_knowledge_check", physics_knowledge_check)
workflow.add_node("math_knowledge_check", math_knowledge_check)
workflow.add_conditional_edges(
    "generate",
    route_topic
)
workflow.set_entry_point("generate")

topic_flow = workflow.compile()
import requests, asyncio
# lock = asyncio.Lock()
# async def auto_label(request_dict: dict) -> dict:
#     async with lock:
#         time.sleep(5)
#         result = {"errCode": 0, "msg": "success", "labels": []}
#         #清洗文本，组装参数
#         sentence = build_params(request_dict)
#         subject_id = request_dict["subject_id"]
#         try:
#             messages = topic_flow.invoke({"topic_content": sentence, "subject_id": subject_id})
#             pred_label = messages["label_value"]
#             if len(pred_label) != 0:
#                 result["labels"] = pred_label.split(",")
#         except Exception as e:
#             log.error(e)
#             result = {"errCode": -1, "msg": "处理失败"}
#         requests.post(request_dict["call_back_url"], json=result)
#     return result
def auto_label(request_dict: dict) -> dict:
    result = {"errCode": "00", "errMsg": "success"}
    #清洗文本，组装参数
    try:
        data = {}
        data["topic_list"] = []
        for topic in request_dict["topic_list"]:
            topic_result = {}
            sentence = build_params(topic)
            subject_id = request_dict["subject_id"]
            messages = topic_flow.invoke({"topic_content": sentence, "subject_id": subject_id})
            pred_label = messages["label_value"]
            if len(pred_label) != 0:
                topic_result["labels"] = pred_label.split(",")
                topic_result["knowsledge_state"] = 1
            else:
                topic_result["labels"] = []
                topic_result["knowsledge_state"] = -1
            topic_result["topic_id"] = topic["topic_id"]
            data["topic_list"].append(topic_result)
        result["data"] = data
    except Exception as e:
        log.error(e)
        result = {"errCode": "-1", "errMsg": "处理失败"}
    return result
